@@ -1,8 +1,10 @@
-from .CARE_projection import CARE_projection
+from .project import *
 from .data import DataProcess
-from .predict import PredictMovie, PredictStack
+from .predict import PredictStack
+from .postprocess import *
 from .train import Trainer
 from .utils import *
+
 # import models
 from .ProjNet import ProjNet
 
