@@ -1,22 +1,31 @@
 from datetime import datetime
 
-import kornia
 import torch.optim as optim
 from barbar import Bar
 from torch.utils.data import DataLoader, random_split
 
+from . import ProjNet
 from .utils import *
 
-device = torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')
+# select device
+if torch.has_cuda:
+    device = torch.device('cuda:0')
+elif hasattr(torch, 'has_mps'):  # only for apple m1/m2/...
+    if torch.has_mps:
+        device = torch.device('mps')
+    else:
+        device = torch.device('cpu')
+else:
+    device = torch.device('cpu')
 
 
 class Trainer:
     """
     Class for training of neural network for surface projection
     """
-    def __init__(self, dataset, num_epochs, network, mode='binary_mask',
-                 n_slices=20, batch_size=4, n_filter=32, lr=1e-4, momentum=0.99, val_split=0.25, save_dir='./',
-                 load_weights=None, save_iterations=False, weights_edge_loss=(1, 0), loss_func='BCEDiceLoss',
+    def __init__(self, dataset, num_epochs, network=ProjNet, mode='binary_mask',
+                 n_slices=20, batch_size=4, n_filter=32, lr=1e-4, val_split=0.25, save_dir='./trained_networks/',
+                 load_weights=None, save_iterations=False, loss_func='BCEDiceLoss',
                  loss_params=(1, 1)):
         """
         Parameters
@@ -46,8 +55,6 @@ class Trainer:
             If not None, load pretrained weights prior to training in path "load_weights"
         save_iterations : bool
             If True, weight of each epoch are save in save_dir
-        weights_edge_loss : tuple(float, float) = (1, 0)
-            Ratio between edge loss and area loss (not clear if useful)
         loss_func : str
             Loss function ('BCEDiceLoss', 'TverskyLoss', 'logcoshTverskyLoss')
         loss_params : tuple(float, float)
@@ -60,7 +67,6 @@ class Trainer:
         self.mode = mode
         self.loss_func = loss_func
         self.loss_params = loss_params
-        self.weights_edge_loss = weights_edge_loss
         self.save_iterations = save_iterations
         self.n_slices = n_slices
         self.batch_size = batch_size
@@ -99,23 +105,15 @@ class Trainer:
                 y_i = batch_i['mask'].view(self.batch_size, 1, self.n_slices, self.dim[0], self.dim[1]).to(device)
                 mult = torch.mul(y_i, x_i)
                 max_i = torch.max(mult, 2)[0]
-                edge = kornia.sobel(y_i.view(self.batch_size, self.n_slices, self.dim[0], self.dim[1])).to(device)
 
                 # Forward pass: Compute predicted y by passing x to the model
-                y_pred, mask_pred, edge_pred = self.model(x_i)
+                y_pred, mask_pred = self.model(x_i)
 
                 # Compute and print loss
                 if self.mode == 'binary_mask':
                     loss = self.criterion(mask_pred, y_i)
                 elif self.mode == 'max_projection':
                     loss = self.criterion(y_pred, max_i)
-                elif self.mode == 'edge':
-                    mask_pred = mask_pred.view(self.batch_size, self.n_slices, self.dim[0], self.dim[1])
-                    y_i = y_i.view(self.batch_size, self.n_slices, self.dim[0], self.dim[1])
-                    edge_pred = edge_pred.view(self.batch_size, self.n_slices, self.dim[0], self.dim[1])
-                    edge = edge.view(self.batch_size, self.n_slices, self.dim[0], self.dim[1])
-                    loss = self.weights_edge_loss[0] * self.criterion(mask_pred, y_i) + \
-                           self.weights_edge_loss[1] * self.criterion(edge_pred, edge)
                 else:
                     raise ValueError(f'Unknown mode {self.mode}.')
 
@@ -133,23 +131,15 @@ class Trainer:
                     y_i = batch_i['mask'].view(self.batch_size, 1, self.n_slices, self.dim[0], self.dim[1]).to(device)
                     mult = torch.mul(y_i, x_i)
                     max_i = torch.max(mult, 2)[0]
-                    edge = kornia.sobel(y_i.view(self.batch_size, self.n_slices, self.dim[0], self.dim[1])).to(device)
 
                     # Forward pass: Compute predicted y by passing x to the model
-                    y_pred, mask_pred, edge_pred = self.model(x_i)
+                    y_pred, mask_pred = self.model(x_i)
 
                     # Compute and print loss
                     if self.mode == 'binary_mask':
                         loss = self.criterion(mask_pred, y_i)
                     elif self.mode == 'max_projection':
                         loss = self.criterion(y_pred, max_i)
-                    elif self.mode == 'edge':
-                        mask_pred = mask_pred.view(self.batch_size, self.n_slices, self.dim[0], self.dim[1])
-                        y_i = y_i.view(self.batch_size, self.n_slices, self.dim[0], self.dim[1])
-                        edge_pred = edge_pred.view(self.batch_size, self.n_slices, self.dim[0], self.dim[1])
-                        edge = edge.view(self.batch_size, self.n_slices, self.dim[0], self.dim[1])
-                        loss = self.weights_edge_loss[0] * self.criterion(mask_pred, y_i) + \
-                               self.weights_edge_loss[1] * self.criterion(edge_pred, edge)
 
                     loss_list.append(loss.to('cpu'))
             val_loss = torch.stack(loss_list).mean()
@@ -162,7 +152,6 @@ class Trainer:
         Start training of DeepProjection
         """
         for epoch in range(self.num_epochs):
-            self.epoch += 1
             self.__iterate('train')
             self.state = {
                 'epoch': self.epoch,
@@ -175,7 +164,6 @@ class Trainer:
                 'n_filter': self.n_filter,
                 'datetime': datetime.now(),
                 'lr': self.lr,
-                'weight_edge_loss': self.weights_edge_loss,
                 'aug_factor': self.dataset.aug_factor,
                 'brightness_contrast': self.dataset.brightness_contrast,
                 'noise_amp': self.dataset.noise_amp,
@@ -186,7 +174,7 @@ class Trainer:
                 'mode': self.dataset.mode,
             }
             val_loss = self.__iterate('val')
-            self.state['loss'][self.epoch] = val_loss.cpu().numpy()
+            self.state['loss'][epoch] = val_loss.cpu().numpy()
             self.scheduler.step(val_loss)
             if val_loss < self.best_loss:
                 print('\nValidation loss improved from %s to %s - saving model state' % (
@@ -194,5 +182,5 @@ class Trainer:
                 self.state['best_loss'] = self.best_loss = val_loss
                 torch.save(self.state, self.save_dir + f'/model_best.pth')
             if self.save_iterations:
-                torch.save(self.state, self.save_dir + f'/model_e{epoch}.pth')
-
+                torch.save(self.state, self.save_dir + f'/model_e{self.epoch}.pth')
+            self.epoch += 1
